# Generated by roxygen2: do not edit by hand

export(tidy_app)
export(tidy_dir)
export(tidy_eval)
export(tidy_file)
export(tidy_pipe)
export(tidy_rstudio)
export(tidy_source)
export(usage)
import(stats)
import(utils)
